# 2.3 【基础】整数与浮点数

Python 支持的数字类型有三种：整数、浮点数和复数。

## 1. 整数（Int）

通常被称为整型，是正或负整数，不带小数点。例如：1，100，-8080，0，等等。

```python
>>> a = 100
>>> type(a)
<class 'int'>
>>>
>>> b = -100
>>> type(b)
<class 'int'>
```

表示数字的时候，有时我们还会用八进制或十六进制来表示：

- 十六进制：用0x前缀和0-9，a-f表示，例如：0xff00，0xa5b4c3d2。

- 八进制：用0o前缀和0-7表示，例如0o12

```python
>>> a = 0x0001 # 十六进制
>>> a
1
>>> type(a)
<class 'int'>
>>> 
>>> b = 0o01  # 八进制
>>> b
1
>>> type(b)
<class 'int'>
```

## 2. 浮点数(Float)

浮点数也就是小数，如1.23，3.14，-9.01，等等。

```python
>>> a = 1.23
>>> a
1.23
>>> type(a)
<class 'float'>
```

之所以称为浮点数，是因为按照科学记数法表示时，一个浮点数的小数点位置是可变的，比如，`1.23x10^9`和`12.3x10^8`是完全相等的。浮点数可以用数学写法，如`1.23`，`3.14`，`-9.01`，等等。但是对于很大或很小的浮点数，就必须用科学计数法表示，把10用e替代，1.23x109就是`1.23e9`，或者`12.3e8`，0.000012可以写成`1.2e-5`，等等。

```python
>>> 1.23e9
1230000000.0
>>> 12.3e8
1230000000.0
```

## 3. 复数(Complex)

复数由实数部分和虚数部分构成，可以用`a + bj`，或者 `complex(a,b)` 表示，复数的实部a和虚部b都是浮点型。关于复数，不做科学计算或其它特殊需要，通常很难遇到，这里不做过多解释。

```python
>>> a = 10 + 0.2j
>>> a
(10+0.2j)
>>> type(a)
<class 'complex'>
>>>
>>> b = complex(10, 0.2)
>>> b
(10+0.2j)
>>> type(b)
<class 'complex'>
```

## 4. 常用方法

**两数运算**

两数相加减

```python
>>> a = 10
>>> b = 20
>>> a + b
30
>>> b - a
10
```

两数相乘除

```python
>>> a = 10
>>> b = 20
>>> a * b
200
>>> b / a
2.0
```

取模取余

```python
>>> a = 10
>>> b = 3
>>> a // b
3
>>> a % b
1
```

**计算绝对值**

```python
>>> a = -10
>>> abs(a)
10
```

**数值直接取整**

```python
>>> a = 3.14
>>> int(a)
3
>>> b = 3.78
>>> int(b)
3
```

**数值四舍五入**

```python
>>> a = 3.14
>>> round(a)
3
>>> b = 3.78
>>> round(b)
4
```

