# 2.6 【基础】运算符（超全整理）

## 0. 什么是运算符？

本章节主要说明Python的运算符。举个简单的例子 **4 +5 = 9** 。 例子中，4和5被称为操作数，"+"号为运算符。

Python语言支持以下类型的运算符:

- 算术运算符
- 比较（关系）运算符
- 赋值运算符
- 逻辑运算符
- 位运算符
- 成员运算符
- 身份运算符
- 运算符优先级

接下来让我们一个个来学习Python的运算符。

------

## 1. Python算术运算符

以下假设变量a为10，变量b为20：

| 运算符 | 描述                                            | 实例                                               |
| :----- | :---------------------------------------------- | :------------------------------------------------- |
| +      | 加 - 两个对象相加                               | a + b 输出结果 30                                  |
| -      | 减 - 得到负数或是一个数减去另一个数             | a - b 输出结果 -10                                 |
| *      | 乘 - 两个数相乘或是返回一个被重复若干次的字符串 | a * b 输出结果 200                                 |
| /      | 除 - x除以y                                     | b / a 输出结果 2                                   |
| %      | 取模 - 返回除法的余数                           | b % a 输出结果 0                                   |
| **     | 幂 - 返回x的y次幂                               | a**b 为10的20次方， 输出结果 100000000000000000000 |
| //     | 取整除 - 返回商的整数部分                       | 9//2 输出结果 4 , 9.0//2.0 输出结果 4.0            |

以下实例演示了Python所有算术运算符的操作

**两数相加减**

```python
>>> a = 10
>>> b = 20
>>> a + b
30
>>> b - a
10
```

**两数相乘除**

```python
>>> a = 10
>>> b = 20
>>> a * b
200
>>> b / a
2.0
```

**取模取余**

```python
>>> a = 10
>>> b = 3
>>> a // b
3
>>> a % b
1
```

## 2. Python比较运算符

以下假设变量a为10，变量b为20：

| 运算符 | 描述                                                         | 实例                  |
| :----- | :----------------------------------------------------------- | :-------------------- |
| ==     | 等于 - 比较对象是否相等                                      | (a == b) 返回 False。 |
| !=     | 不等于 - 比较两个对象是否不相等                              | (a != b) 返回 true.   |
| >      | 大于 - 返回x是否大于y                                        | (a > b) 返回 False。  |
| <      | 小于 - 返回x是否小于y。所有比较运算符返回1表示真，返回0表示假。这分别与特殊的变量True和False等价。注意，这些变量名的大写。 | (a < b) 返回 true。   |
| >=     | 大于等于 - 返回x是否大于等于y。                              | (a >= b) 返回 False。 |
| <=     | 小于等于 - 返回x是否小于等于y。                              | (a <= b) 返回 true。  |

以下实例演示了Python所有比较运算符的操作：

判断两数是否相等

```python
>>> a = 1
>>> b = 1
>>> a == b
True
>>> a != b
False
```

判断 a 是否大于 b

```python
>>> a = 1
>>> b = 2
>>> a > b
False
>>> a < b
True
```

## 3. Python赋值运算符

以下假设变量a为10，变量b为20：

| 运算符 | 描述             | 实例                                  |
| :----- | :--------------- | :------------------------------------ |
| =      | 简单的赋值运算符 | c = a + b 将 a + b 的运算结果赋值为 c |
| +=     | 加法赋值运算符   | c += a 等效于 c = c + a               |
| -=     | 减法赋值运算符   | c -= a 等效于 c = c - a               |
| *=     | 乘法赋值运算符   | c *= a 等效于 c = c * a               |
| /=     | 除法赋值运算符   | c /= a 等效于 c = c / a               |
| %=     | 取模赋值运算符   | c %= a 等效于 c = c % a               |
| **=    | 幂赋值运算符     | c **= a 等效于 c = c ** a             |
| //=    | 取整除赋值运算符 | c //= a 等效于 c = c // a             |

以下实例演示了Python所有赋值运算符的操作：

这里只以加减为例，其它的同理

```python
>>> a = 1
>>> a += 2
>>> a
3
>>> a += 2
>>> a
5
>>> a -= 2
>>> a
3
```

## 4. Python位运算符

按位运算符是把数字看作二进制来进行计算的。Python中的按位运算法则如下：

| 运算符 | 描述           | 实例                                                         |
| :----- | :------------- | :----------------------------------------------------------- |
| &      | 按位与运算符   | (a & b) 输出结果 12 ，二进制解释： 0000 1100                 |
| \|     | 按位或运算符   | (a \| b) 输出结果 61 ，二进制解释： 0011 1101                |
| ^      | 按位异或运算符 | (a ^ b) 输出结果 49 ，二进制解释： 0011 0001                 |
| ~      | 按位取反运算符 | (~a ) 输出结果 -61 ，二进制解释： 1100 0011， 在一个有符号二进制数的补码形式。 |
| <<     | 左移动运算符   | a << 2 输出结果 240 ，二进制解释： 1111 0000                 |
| >>     | 右移动运算符   | a >> 2 输出结果 15 ，二进制解释： 0000 1111                  |

以下实例演示了Python所有位运算符的操作：

与运算

```python
>>> a = 60  # 60 = 0011 1100 
>>> b = 13  # 13 = 0000 1101 
>>> c = 0
>>>  a & b  # 12 = 0000 1100
12
```

或运算

```python
>>> a | b  #  61 = 0011 1101 
61
```

异或运算

```python
>>> a ^ b  # 49 = 0011 0001
49
```

取反运算

```python
>>> a = 60  # 60 = 0011 1100 
>>> ~a        # -61 = 1100 0011
-61
```

左移动运算符

```python
>>> a = 60  # 60 = 0011 1100 
>>> a << 2  # 240 = 1111 0000
240
```

右移动运算符

```python
>>> a = 60  # 60 = 0011 1100 
>>> a >> 2  # 15 = 0000 1111
15
```

## 5. Python逻辑运算符

Python语言支持逻辑运算符，以下假设变量 a 为 10, b为 20:

| 运算符 | 逻辑表达式 | 描述                                                         | 实例                    |
| :----- | :--------- | :----------------------------------------------------------- | :---------------------- |
| and    | x and y    | 布尔"与" - 如果 x 为 False，x and y 返回 False，否则它返回 y 的计算值。 | (a and b) 返回 20。     |
| or     | x or y     | 布尔"或" - 如果 x 是非 0，它返回 x 的值，否则它返回 y 的计算值。 | (a or b) 返回 10。      |
| not    | not x      | 布尔"非" - 如果 x 为 True，返回 False 。如果 x 为 False，它返回 True。 | not(a and b) 返回 False |

以上实例输出结果：

and：必须都为True，才能返回True，否则返回False

```python
>>> True and True
True
>>> True and False
False
```

or：只要有一个为True，就返回True，其他返回False

```python
>>> True or False
True
>>> True or True
True
```

not：与原值取反

```python
>>> not True
False
>>> not False
True
```

## 6. Python成员运算符

除了以上的一些运算符之外，Python还支持成员运算符，测试实例中包含了一系列的成员，包括字符串，列表或元组。

| 运算符 | 描述                                                  | 实例                                        |
| :----- | :---------------------------------------------------- | :------------------------------------------ |
| in     | 如果在指定的序列中找到值返回True，否则返回False。     | x 在 y序列中 , 如果x在y序列中返回True。     |
| not in | 如果在指定的序列中没有找到值返回True，否则返回False。 | x 不在 y序列中 , 如果x不在y序列中返回True。 |

以下实例演示了Python所有成员运算符的操作：

```python
>>> "Apple" in ["Apple", "Huawei"]
True
>>>
>>> "Vivo" not in ["Apple", "Huawei"]
True
```

## 7. Python身份运算符

身份运算符用于比较两个对象的存储单元

| 运算符 | 描述                                       | 实例                                                       |
| :----- | :----------------------------------------- | :--------------------------------------------------------- |
| is     | is是判断两个标识符是不是引用自一个对象     | x is y, 如果 id(x) 等于 id(y) , **is** 返回结果 1          |
| is not | is not是判断两个标识符是不是引用自不同对象 | x is not y, 如果 id(x) 不等于 id(y). **is not** 返回结果 1 |

以下实例演示了Python所有身份运算符的操作：

```python
>>> a = 1000
>>> b = 1000
>>> id(a)
4532310864
>>> id(b)
4532311632
>>> a is b
False
```

## 8. Python运算符优先级

以下表格列出了从最高到最低优先级的所有运算符：

| 运算符                   | 描述                                                   |
| :----------------------- | :----------------------------------------------------- |
| **                       | 指数 (最高优先级)                                      |
| ~ + -                    | 按位翻转, 一元加号和减号 (最后两个的方法名为 +@ 和 -@) |
| * / % //                 | 乘，除，取模和取整除                                   |
| + -                      | 加法减法                                               |
| >> <<                    | 右移，左移运算符                                       |
| &                        | 位 'AND'                                               |
| ^ \|                     | 位运算符                                               |
| <= < > >=                | 比较运算符                                             |
| <> == !=                 | 等于运算符                                             |
| = %= /= //= -= += *= **= | 赋值运算符                                             |
| is is not                | 身份运算符                                             |
| in not in                | 成员运算符                                             |
| not or and               | 逻辑运算符                                             |

以下实例演示了Python运算符优先级的操作：

```python
>>> 20 + ( 30 * 2 ) / 5
32.0
```

计算顺序是

1. 30*2 = 60
2. 60/5 = 12.0
3. 20 + 12.0 = 32.0

