# 4.1 【基础】条件语句：if 

## 1. 简单小例子

如果满足条件 A，则执行代码块 a，否则执行代码块 b。类似这样的控制流程语句，称之为条件语句。

它的基本形式是

```python
if 判断条件：
    执行语句……
else：
    执行语句……
```

举个最简单的例子

```python
>>> age = 20
>>> if age >=18:
...     print("已经是成年人")
... else:
...     print("还是未成年人")
...
已经是成年人
```

## 2. 多条件语句

如果需要多次判断可以利用 `elif`，它的基本形式是

```python
if 判断条件1：
    执行语句……
elif 判断条件2：
    执行语句……
elif 判断条件3：
    执行语句……
else:
    执行语句……
```
举个最简单的例子

```python
>>> score = 75
>>>
>>> if score >=90:
...     print("优秀")
... elif score >= 80:
...     print("良好")
... elif score >= 70:
...     print("一般")
... elif score >= 60:
...     print("合格")
... else:
...     print("不合格")
...
一般
```

## 3. 判断的条件

在 Python 中，值可以分为

- `假值` ：None、空列表、空集合、空字典，空元组、空字符串、0、False 等
- `真值` ：非空列表、非空集合、非空字典，非空元组、非空字符串、非 0 数值、True 等

`if` 和 `elif` 后面可以接一个表达式（上面已经举例过），也可以接一个对象。

只要这个对象是真假，代码就会进入相应分支，如果为对象为假值，则继续下一判断。

这边随便以 0 和 1 举例

```python
>>> aint = 1
>>> if aint:
...     print("ok")
...
ok
```

## 4. 多个条件组合

在讲多个条件组合时，先来了解一下 Python 中的逻辑运算符。

以下假设变量 a 为 10, b为 20:
| 运算符 | 逻辑表达式 | 描述| 实例                    |
| --- | --- | --- | --- |
| and    | x and y    | 布尔"与" - 如果 x 为 False，x and y 返回 False，否则它返回 y 的计算值。 | (a and b) 返回 20。     |
| or     | x or y     | 布尔"或" - 如果 x 是非 0，它返回 x 的值，否则它返回 y 的计算值。 | (a or b) 返回 10。      |
| not    | not x      | 布尔"非" - 如果 x 为 True，返回 False 。如果 x 为 False，它返回 True。 | not(a and b) 返回 False |



学习完逻辑运算符，就可以开始写多条件语句

- 如果一个 if 条件需要同时满足多个条件，那么可以使用 `and`  或者 `&`
- 如果一个 if 条件只需要满足多个条件的其中一个，那么可以使用 `or` 或者 `|`
- 如果一个 if 要求不满足某个条件，那么可以使用 `not` 或者 `!`

```python
# 需要同时满足条件 A 和条件 B
if 条件A and 条件B:
	...
  
# 只需要满足条件A或者条件B即可
if 条件A or 条件B:
	...
  
# 要求不满足条件A
if not 条件A:
	...
```

