# 5.3 【基础】匿名函数的使用

匿名函数（英语：anonymous function）是指一类无需定义标识符（函数名）的函数。通俗来说呢，就是它可以让我们的函数，可以不需要函数名。

正常情况下，我们定义一个函数，使用的是 `def` 关键字，而当你学会使用匿名函数后，替代 `def` 的是 `lambda`。

这边使用`def` 和 `lambda` 分别举个例子，你很快就能理解。

```python
def mySum(x, y):
    return x+y
mySum(2, 3)
# 5

(lambda x, y: x+y)(2, 4)
# 6
```

从上面的示例，我们可以看到匿名函数直接运行，省下了很多行的代码，有没有？

接下来，我们的仔细看一下它的用法

带 if/else

```python
>>>( lambda x, y: x if x < y else y )( 1, 2 )
1
```

嵌套函数

```python
>>>( lambda x: ( lambda y: ( lambda z: x + y + z  )( 1 ) )( 2 ) )( 3 )
6
```

递归函数

```python 
>>> func = lambda n:1 if n == 0 else n * func(n-1)
>>> func(5)
120
```

或者

```python
>>> f = lambda func, n: 1 if n == 0 else n * func( func, n - 1 )
>>> f(f,4)
24
```

从以上示例来看，lambda 表达式和常规的函数相比，写法比较怪异，可读性相对较差。除了可以直接运行之外，好像并没有其他较为突出的功能，为什么在今天我们要介绍它呢？

首先我们要知道 lambda 是一个表达式，而不是一个语句。正因为这个特点，我们可以在一些特殊的场景中去使用它。具体是什么场景呢？接下来我们会介绍到几个非常好用的内置函数。
