# 5.4 【基础】必学高阶函数

## 1. map 函数

map 函数，它接收两个参数，第一个参数是一个函数对象（当然也可以是一个lambda表达式），第二个参数是一个序列。

它可以实现怎样的功能呢，我举个例子你就明白了。

```python
>>> map(lambda x: x*2, [1,2,3,4,5])
[2, 4, 6, 8, 10]
```

可以很清楚地看到，它可以将后面序列中的每一个元素做为参数传入lambda中。

当我们不使用 map 函数时，你也许会这样子写。

```python
mylist=[]
for i in [1,2,3,4,5]:
    mylist.append(i*2)
```

## 2. filter 函数

filter 函数，和 map 函数相似。同样也是接收两个参数，一个lambda 表达式，一个序列。它会遍历后面序列中每一个元素，并将其做为参数传入lambda表达式中，当表达式返回 True，则元素会被保留下来，当表达式返回 False ，则元素会被丢弃。

下面这个例子，将过滤出一个列表中小于0的元素。

```python
>>>filter(lambda x: x < 0, range(-5, 5))
[-5, -4, -3, -2, -1]
```

## 3. reduce 函数

reduce 函数，也是类似的。它的作用是先对序列中的第 1、2 个元素进行操作，得到的结果再与第三个数据用 lambda 函数运算，将其得到的结果再与第四个元素进行运算，以此类推下去直到后面没有元素了。

![](http://image.iswbm.com/20200930175131.png)

这边举个例子你也就明白了。

```python
>>>reduce(lambda x,y: x+y, [1,2,3,4,5])
15
```

它的运算过程分解一下是这样的。

```python
1+2=3
3+3=6
6+4+10
10+5=15
```

## 4. 注意点

以上几个函数，熟练的掌握它们的写法，可以让我们的代码看起来更加的 Pythonic ，在某一程度上代码看起来更加的简洁。

如果你是新手呢，你需要注意的是，以上示例是在 Python2.x 环境下演示的。而在 Python3.x 中，却有所不同，你可以自己尝试一下。

这里总结一下：

第一点，map 和 filter 函数返回的都不再是一个列表，而是一个迭代器对象。这里以map为例

```python
>>> map_obj = map(lambda x: x*2, [1,2,3,4,5])
>>> from collections.abc import Iterator
>>> isinstance(map_obj, Iterator)
True
>>> next(map_obj)
2
>>> list(map_obj)
[4, 6, 8, 10]
```

第二点，reduce 不可以直接调用，而是要先导入才能使用，

```python
from functools import reduce
```

