# 12.4 【虚拟环境】方案三：使用 pipx

## 1. 什么是 pipx

pipx 是一款用于帮助你安装和运行那些用 python 编写的终端程序，它类似于 macOS 上的 brew，Ubuntu 上的 apt，CentOS 上的 yum。

pipx 依赖 pip 和 venv，它只能在 python 3.6+ 的 Python 版本中才能使用。

默认情况下，pipx 和 pip 一样会从 pypi 上安装包，同时 pipx 也能像 pip 一样从本地、git仓库、wheel 文件中安装包。

为了避免你在安装 python app时，由于多版本而导致冲突，通常我们会使用 venv 或者 virtualenv 新建一个虚拟环境，然后将 app 安装到虚拟环境中。

后续你对这个 app 的管理操作，都得先进入这个虚拟环境。

发现没有？好像有点麻烦。

pipx 的存在使这个流程变得更加舒畅，使用 pipx 你可以无需关注虚拟环境的存在，并在你的机器上安装多个版本的 python app。

## 2. 安装使用

安装 pipx

```shell
$ python3 -m pip install --user pipx
$ python3 -m userpath append ~/.local/bin
Success!
```

使用 Pipx 需要注意两个路径

1. 二进制文件的保存位置：默认是 `~/.local/bin`，可使用环境变量 `PIPX_BIN_DIR` 进行更改，或者执行如下命令(`python3 -m userpath append ${you_path}`)
2. 虚拟环境的保存位置：默认是 `~/.local/pipx`，可使用环境变量 `PIPX_HOME` 进行更改

在我安装好 pipx ，准备使用的时候，发现全局找不到 pipx 这个命令。

![](http://image.iswbm.com/image-20201130124107950.png)

按照如上图所示，难道使用全路径执行命令？

**不，怎么都觉得不太对劲。。**

想要解决这个问题，其实很简单，有两种方法（两种都可以，我演示使用的第一种方法）：

1. 添加个软链接指向刚刚那个全路径就好啦
2. 将这个路径添加到 PATH 中 `/Users/MING/Library/Python/3.9/bin/ `

```shell
$ ln -s /Users/MING//Library/Python/3.9/bin/pipx /usr/local/bin/pipx
```

软链接建好后，就可以直接使用 `pipx` 的命令啦。

![](http://image.iswbm.com/image-20201130124554404.png)

刚刚我使用 pipx 安装了 youtube-dl 后，其实并没有将这个 youtube-dl 安装到系统全局的 Python 环境中。

还记得最开始，我强调过两个非常重要的路径吗？

现在来看一下，这个路径下面都有哪些东西？

![](http://image.iswbm.com/image-20201130125257203.png)

从截图上可以看出

- pipx 在 `~/.local/pipx/venvs` 目录下新建了个名叫 `youtube-dl ` 的虚拟机环境
- 并将 `youtube-dl ` 安装到这个虚拟机环境中
- 然后在 `~/.local/bin` 的目录下新建一个软链接，指向这个虚拟环境中
- 这样 `youtube-dl ` 就变成全局的工具啦。

![](http://image.iswbm.com/image-20201130131138939.png)

为了避免你新安装的 youtube-dl 与全局的冲突，你也可以指定 pipx 的命令来运行 youtube-dl 

```shell
$ pipx run youtube-dl --no-check-certificate https://www.bilibili.com/video/BV1jK4y1h7uA
```

运行效果如下：

![](http://image.iswbm.com/image-20201130210539907.png)

pip run 也可以直接执行在线的 python 脚本

```shell
$ pipx run https://gist.githubusercontent.com/cs01/fa721a17a326e551ede048c5088f9e0f/raw/6bdfbb6e9c1132b1c38fdd2f195d4a24c540c324/pipx-demo.py
pipx is working!
```

## 3. 查看包

查看已安装过的包

```shell
$ pipx list
```

## 4. 安装包

下载最新版本的 python 包，并安装到新建的虚拟环境中

```shell
$ pipx install <PACKAGE>
```

## 4. 运行APP

`pipx run` 后面可接一个包的 url 链接，会将这个包下载下来并运行，也可以接已安装过的应用名来直接运行它

```shell
$ pipx run <PACKAGE_URL/APP>
```

如果一个 app 有多个版本，那么可以通过 `spec` 指定版本号

```shell
$ pipx run --spec PACKAGE==1.0.0 app
```

更神奇的是，pipx 支持指定 git 代码仓库直接运行

```shell
$ pipx run --spec git+https://github.com/psf/black.git black

# 指定分支
$ pipx run --spec git+https://github.com/psf/black.git@branch black  

# 指定某个git hash
$ pipx run --spec git+https://github.com/psf/black.git@ce14fa8b497bae2b50ec48b3bd7022573a59cdb1 black 

# 指定某个发行版本
$ pipx run --spec https://github.com/psf/black/archive/18.9b0.zip black # install a release
```

## 5. 升级包

升级某个包

```shell
$ pipx upgrade <pkg>
```

升级全部包

```shell
$ pipx upgrade-all
```

## 6. 卸载包

卸载某个包

```shell
$ pipx uninstall <pkg>
```

卸载全部包

```shell
$ pipx uninstall-all
```

重装全部包

```shell
$ pipx reinstall-all
```

## 7. 使用 pip

每执行一次 pipx install 就会新建一个虚拟环境，那我们有没有办法管理这些虚拟机环境呢？

比如我想看这个虚拟环境里安装了哪些包？

使用如下命令就可以像使用 pip 一样，来管理 pipx 的虚拟环境

```shell
$ pipx runpip <env_name> <args>
```

效果如下

![](http://image.iswbm.com/image-20201130215320069.png)

## 8. 其他

执行 `pipx completions` 可以启用 pipx 的补全说明。

对于不同的终端开启方式不一样，我使用的是 zsh，方法是

```shell
$ autoload -U bashcompinit
$ bashcompinit
$ eval "$(register-python-argcomplete pipx)"
```

我安装好后，可以使用 tab 键进行命令补全。

![](http://image.iswbm.com/image-20201130220233001.png)

执行 `pipx ensurepath` 可以确保 `~/.local/bin` 这个重要的目录，已经放入到 `$PATH` 的变量中。

![](http://image.iswbm.com/image-20201130215826513.png)

##  9. pipx vs pip

pipx 只是解决 pip 的一个痛点，因此他的适用场景比较单一，它只适用于安装和运行那些有提供命令行入口的app。

- pip 适用于大多数的 Python 版本，而 pipx 需要 Python 3.6+ 才可以使用
- pipx 依赖 pip 和 venv，可以使用 pip 安装pipx ，反过来则不行。
- pip 和 pipx 默认都是从 pypi 上安装包
- pipx 在安装和管理 cli 应用程序时，比 pip 更灵活，它可以在允许在隔离环境中安装和运行 Python 应用



## 10. 参考文章

- https://github.com/pipxproject/pipx
- https://pipxproject.github.io/pipx/comparisons/